/***MEM:S***/
/*
Sea O(N^2*logN + M) implementation. O(N) Memory.

by Radu Berinde
*/

#pragma option -f287

#ifdef __BORLANDC__
#pragma option -3 -O2
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define sqr(x) ((x)*(x))

#define VSIZE 401
#define TLEV  10
#define TSIZE (1 << TLEV)
#define TSTART (1 << (TLEV-1))
#define EPS 1e-4
#define XSentinel 100010.0

typedef struct vessel
{
    double x, y;
} vessel;

int Mirrored;

vessel V[VSIZE];
int N;
long M;

double Treex[TSIZE];
int   Treev[TSIZE];

int VList[VSIZE];
int ListSize;
int VPos[VSIZE];

double CurrX;
FILE *fi, *fo;

void setvalue(int i, double x)
{
    int p;
    Treev[TSTART + i] = i, Treex[TSTART + i] = x;
    for (i = (i + TSTART)/2; i; i /= 2)
    {
        p = (Treex[2*i] <= Treex[2*i+1]) ? 2*i : 2*i+1;
        if (Treev[i] == Treev[p] && Treex[i] == Treex[p]) return;
        Treev[i] = Treev[p], Treex[i] = Treex[p];
    }
}

void init_tree()
{
    int i;
    for (i = 0; i < TSIZE-TSTART; i++)
        setvalue(i, XSentinel);
}

double computeposition(int i)
{
    vessel u, v;
    double dx, nx;
    
    if (i + 1 == ListSize) return XSentinel;
    
    u = V[VList[i]];
    v = V[VList[i+1]];
    if (u.x > v.x) return XSentinel;
    if (u.y > v.y)
       printf("This shouldn\'t happen to a dog\n");
    dx = v.x - u.x;
    nx = (double) v.x + (v.y*(double) v.y - u.y*(double) u.y - dx*dx) / (2.0 * dx);
    if (nx < CurrX - EPS) printf("This shouldn\'t happen to a dog either.\n");

    return (nx > XSentinel) ? XSentinel : nx;
}

void updateposition(int i)
{
    if (i >= 0 && i < ListSize)
       setvalue(VList[i], computeposition(i));
}

void listswap(int i)
{
    int t;
    t = VList[i], VList[i] = VList[i+1], VList[i+1] = t;
    VPos[VList[i]] = i;
    VPos[VList[i+1]] = i+1;

    updateposition(i-1);
    setvalue(VList[i], XSentinel);
    updateposition(i+1);
}

#define ldist(xx, v) (sqr((xx) - V[v].x) + sqr(V[v].y))

void lightprint(double x, int n)
{
    fprintf(fo, "%.4f\n", sqrt(sqr(x - V[VList[n-1]].x) + sqr(V[VList[n-1]].y)));
}

int cmpv(const void *a, const void *b)
{
    return ((vessel *) a)->x < ((vessel *) b)->x ? -1 : 1;
}

double Lx;
int Ln;

long currlight = -1;

void nextlight()
{
    if (++currlight < M)
       fscanf(fi, "%lf %d", &Lx, &Ln);
}

void solve()
{
    int i, j, addvesselev;

    init_tree();
    qsort(V, N, sizeof(vessel), cmpv);
    for (i = 1; i < N; i++)
        if (fabs(V[i].x - V[i-1].x) < 0.00005)
           break;
    if (i < N)
       printf("Warning: Duplicate X in lights\n");

    nextlight();
    for (ListSize = 0; currlight < M;)
    {
        CurrX = Treex[1];
        if (ListSize < N && V[ListSize].x < CurrX)
            CurrX = V[ListSize].x, addvesselev = 1;
        else
            addvesselev = 0;

        for (; currlight < M && Lx < CurrX; nextlight())
            lightprint(Lx, Ln);

        if (addvesselev) /* add vessel */
        {
            for (i = 0; i < ListSize; i++)
                if (ldist(CurrX, VList[i]) > ldist(CurrX, ListSize))
                   break;
            for (j = ListSize; j > i; j--)
                VPos[VList[j] = VList[j-1]] = j;
            VPos[VList[i] = ListSize] = i;
            ListSize++;
            updateposition(i-1);
            updateposition(i);
            continue;
        }
        if (Treex[1] < XSentinel && Treev[1] < N) /* special sentinel */
           listswap(VPos[Treev[1]]); /* swap two vessels */
    }
}

void read_data()
{
    int i;
    fi = fopen("sea.in", "rt");
    fo = fopen("sea.out", "wt");
    fscanf(fi, "%d %ld", &N, &M);
    for (i = 0; i < N; i++)
        fscanf(fi, "%lf %lf", &V[i].x, &V[i].y);
}

int main()
{
    read_data();
    solve();
    fclose(fi);
    fclose(fo);
    return 0;
}
